# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('server_guardian', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ServerLog',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('content', models.TextField(verbose_name='content', blank=True)),
                ('time_logged', models.DateTimeField(verbose_name='time_logged')),
                ('status', models.CharField(max_length=16, verbose_name='status', choices=[(b'OK', b'OK'), (b'WARNING', b'WARNING'), (b'DANGER', b'DANGER')])),
                ('label', models.CharField(max_length=512, verbose_name='label')),
            ],
            options={
                'ordering': ('server', 'label', '-time_logged'),
            },
        ),
        migrations.AddField(
            model_name='server',
            name='log_age',
            field=models.PositiveIntegerField(default=10, help_text='How many days the logs are kept. Set to 0 for infinite.', verbose_name='log age'),
        ),
        migrations.AddField(
            model_name='server',
            name='logging',
            field=models.BooleanField(default=False, help_text='Enable logging results to DB (recommended)', verbose_name='logging'),
        ),
        migrations.AddField(
            model_name='serverlog',
            name='server',
            field=models.ForeignKey(related_name='server_logs', verbose_name='server', to='server_guardian.Server'),
        ),
    ]



"""Tests for the models of the server_guardian app."""
from django.test import TestCase
from mixer.backend.django import mixer
from server_guardian import constants

from ..models import get_parsed_response


class ServerTestCase(TestCase):
    """Tests for the ``Server`` model class."""
    longMessage = True

    def test_instantiation(self):
        server = mixer.blend('server_guardian.Server')
        self.assertTrue(server.pk)


class ServerLogTestCase(TestCase):
    """Tests for the ``ServerLog`` model class."""
    longMessage = True

    def test_instantiation(self):
        server_log = mixer.blend('server_guardian.ServerLog')
        self.assertTrue(server_log.pk)

    def test_get_previous(self):
        previous_server_log = mixer.blend(
            'server_guardian.ServerLog')
        mixer.blend(
            'server_guardian.ServerLog',
            label=previous_server_log.label,
        )
        mixer.blend(
            'server_guardian.ServerLog',
            server=previous_server_log.server,
        )
        server_log = mixer.blend(
            'server_guardian.ServerLog',
            label=previous_server_log.label,
            server=previous_server_log.server,
        )
        self.assertEqual(
            server_log.get_previous(),
            previous_server_log,
            msg=(
                'get_previous should return the previous server log from the'
                ' same server and label.'
            )
        )


class GetParsedResponseTestCase(TestCase):
    """Tests for the ``get_parsed_response`` function."""
    longMessage = True

    def test_get_parsed_response(self):
        json_response = (
            '[{"label": "foobar", "status": "OK", "info": "it is OK"}]'
        )
        self.assertEqual(
            get_parsed_response(json_response)[0]['status'],
            constants.SERVER_STATUS['OK'],
            msg=(
                'Should return a status of OK. Info: {0}'.format(
                    get_parsed_response(json_response)
                )
            )
        )
        self.assertEqual(
            get_parsed_response(json_response)[0]['info'],
            'it is OK',
            msg=(
                'It should be possible to read the info from the'
                ' response dict.'
            )
        )
        html_response = '<html><head></head><body>HTML RESPONSE</body></html>'
        self.assertEqual(
            get_parsed_response(html_response)[0]['status'],
            constants.SERVER_STATUS['DANGER'],
            msg=(
                'For an HTML response, the dict should return a'
                ' warning status.'
            )
        )



"""The command, that fetches the information from the configured servers."""
import os
import sys

from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils.timezone import now, timedelta

import requests
from django_libs.utils.email import send_email
from django_libs.decorators import lockfile

from ... import default_settings
from ...models import Server, ServerLog, get_parsed_response
from ...constants import SERVER_STATUS

LOCKFILE = os.path.join(settings.LOCKFILE_FOLDER, 'guardian_fetch')


class Command(BaseCommand):

    def send_error_email(self, serverlog):
        context = {
            'server': serverlog.server,
            'subject': u'{0} {1} - {2}'.format(
                serverlog.status, serverlog.status_code, serverlog.server.name
            ),
            'status': serverlog,
        }
        send_email(
            {},
            context,
            'server_guardian/email/email_subject.html',
            'server_guardian/email/warning_email_body.html',
            settings.FROM_EMAIL,
            [admin[1] for admin in settings.ADMINS],
        )

    def get_email_required(self, serverlog):
        """Send a warning email, if the server response contains errors."""

        # when the status code is included in our setting
        if serverlog.status_code in default_settings.EMAIL_ON_STATUS:
            return True

        try:
            # when the server transitions to a status of DANGER
            if serverlog.status == SERVER_STATUS['DANGER'] and (
                serverlog.get_previous().status != SERVER_STATUS['DANGER']
            ):
                return True

            # when the server transitions to a status that is not DANGER
            if serverlog.status != SERVER_STATUS['DANGER'] and (
                serverlog.get_previous().status == SERVER_STATUS['DANGER']
            ):
                return True
        except (KeyError, TypeError) as ex:
            serverlog.content = (
                'Server got an error "{0}" and returned the following content:'
                '{1}'.format(ex, serverlog.content)
            )
            serverlog.save()
            return True

        return False

    @lockfile(LOCKFILE)
    def handle(self, *args, **options):
        count = 0
        mails_sent = 0

        for server in Server.objects.all():
            response = requests.get(url=server.url, params={
                'token': server.token
            })
            server.last_updated = now()
            server.save()
            parsed_response = get_parsed_response(response.content)
            for status in parsed_response:
                if type(status) == dict:
                    serverlog = ServerLog.objects.create(
                        server=server,
                        content=status['info'],
                        status=status['status'],
                        label=status.get('label', 'no label'),
                        status_code=response.status_code,
                    )
                    if self.get_email_required(serverlog):
                        self.send_error_email(serverlog)
                        mails_sent += 1
                else:
                    self.send_error_email(serverlog)
                    mails_sent += 1
            if server.log_age:
                ServerLog.objects.filter(
                    time_logged__lt=now() - timedelta(days=server.log_age)
                ).delete()
            count += 1

        sys.stdout.write(
            '[{0}] Updated {1} servers. Sent {2} warning emails.\n'.format(
                now(), count, mails_sent)
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('server_guardian', '0003_auto_20160222_0343'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='server',
            name='logging',
        ),
    ]



"""Models for the server_guardian app."""
import json

from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from . import constants


def get_parsed_response(response):
    """Parses the complete response of a server from json."""
    try:
        response = json.loads(response)
    except ValueError:
        response = constants.HTML_STATUS_FALLBACK
    if type(response) != list:
        response = constants.HTML_STATUS_FALLBACK
    return response


@python_2_unicode_compatible
class Server(models.Model):
    name = models.CharField(
        max_length=256,
        blank=True,
        verbose_name=_('server name'),
    )
    url = models.URLField(
        verbose_name=_('API URL'),
        help_text='The URL, that the guardian API is available under.'
    )
    token = models.CharField(
        max_length=256,
        verbose_name=_('token'),
        help_text=(
            'Add this to your client server settings as'
            ' "SERVER_GUARDIAN_SECURITY_TOKEN".'
        ),
    )
    last_updated = models.DateTimeField(
        verbose_name=_('last updated'),
        blank=True, null=True,
    )
    log_age = models.PositiveIntegerField(
        verbose_name=_('log age'),
        help_text=_('How many days the logs are kept. Set to 0 for infinite.'),
        default=10,
    )

    def __str__(self):  # pragma: no cover
        if self.name:
            return self.name
        else:
            return self.url

    def get_absolute_url(self):
        return reverse('server_guardian_dashboard')

    def get_latest_log(self):
        return self.server_logs.all()[:10]


@python_2_unicode_compatible
class ServerLog(models.Model):
    """Stores logging information for a server."""
    content = models.TextField(
        verbose_name=_('content'),
        blank=True,
    )
    server = models.ForeignKey(
        Server,
        verbose_name=_('server'),
        related_name='server_logs',
    )
    time_logged = models.DateTimeField(
        verbose_name=_('time_logged'),
        auto_now=True,
    )
    status = models.CharField(
        verbose_name=_('status'),
        choices=constants.SERVER_STATUS_CHOICES,
        max_length=16,
    )
    status_code = models.CharField(
        verbose_name=_('status code'),
        max_length=3,
    )
    label = models.CharField(
        verbose_name=_('label'),
        max_length=512,
    )

    class Meta:
        ordering = ('server', 'label', '-time_logged')

    def __str__(self):  # pragma: no cover
        return '[{0}] {1} ({2})'.format(
            self.server, self.label, self.time_logged
        )

    def get_previous(self):
        try:
            return self._meta.model.objects.filter(
                label=self.label,
                server=self.server,
                time_logged__lt=self.time_logged,
            )[0]
        except IndexError:
            return self._meta.model(
                status_code=200,
                status=constants.SERVER_STATUS['OK'],
                label=self.label,
                server=self.server,
                content='No previous entry',
            )

    def has_errors(self):
        return self.status in constants.ERROR_STATUS




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('server_guardian', '0004_remove_server_logging'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='server',
            name='response_body',
        ),
        migrations.RemoveField(
            model_name='server',
            name='status_code',
        ),
        migrations.AddField(
            model_name='serverlog',
            name='response_body',
            field=models.TextField(verbose_name='server response', blank=True),
        ),
        migrations.AddField(
            model_name='serverlog',
            name='status_code',
            field=models.CharField(default='200', max_length=3, verbose_name='status code'),
            preserve_default=False,
        ),
    ]



"""URLs for the server_guardian app."""
from django.conf.urls import url

from . import views


urlpatterns = [
    url(r'^$',
        views.GuardianDashboardView.as_view(),
        name='server_guardian_dashboard'),
    url(r'^reload/$',
        views.GuardianReloadView.as_view(),
        name='server_guardian_reload'),
]
